;Macrovision blanker (Despoiler) V1.00b
;Written for an AT90S2313 running at 10MHz
;15th September 1999, Lee Atkins.

 .include "io2313.h"

 ;port d
VSYNC  equ 6        ;Vertical sync in.
HSYNC  equ 5        ;Horizontal sync in.
BRIGHT equ 4        ;Pixel brightness out.

 ;port b
BLANKOUT equ 0      ;Force video out.

Lookup equ $60       ;Where to create the message lookup table.

 .org RESET_vect
 rjmp mainprog

 .org INT1_vect
 reti                 ;Just wake up and continue. (4 clock cycles 400ns@10MHz)

 .org ROMSTART

; **************
; * Chip setup *
; **************
mainprog:
 ldi R16,$DF
 out SP,R16         ;load stack pointer.

 ldi R16,16
 out DDRD,R16        ;port d
 com R16
 out PORTD,R16      ;pull inputs high

 ldi R16,$03
 out DDRB,R16        ;port b setup

 ldi R16,$06
 out TCCR1B,R16       ;Set up timer to count video lines.

 ldi R16,$28          ;int on edge of hsync (falling = $28, rising = $2C)
 out MCUCR,R16        ;set up sleep and interrupt triggers.
 ldi R16,$80
 out GIMSK,R16

 cbi PORTB,BLANKOUT
 clr R16
 ldi R21,48             ;value to convert from ascii

 ldi R28,Lookup             ;Load Y with table start
 clr R29
 ldi R30,(message*2)&255       ;Load Z with start of message.
 ldi R31,(message*2)>8
makelook:
 lpm
 ld R1,Z+                    ;Get a character of message.
 tst R0                      
 breq donelookup             ;finish setup if end of message string.

 sub R0,R21                   ;Convert from ascii
 
 clr R1                     ;Multiply by 8
 rol R0
 rol R1
 rol R0
 rol R1
 rol R0
 rol R1

 st Y+,R0                 ;Store lookup address in ram.
 st Y+,R1
 rjmp makelook

donelookup:
 st Y+,R21                   ;Mark end of lookup with some 48's
 st Y+,R21

 ldi R22,1                   ;Set frame counter

 ldi R17,$08                 ;set up watchdog, 16ms.
 out WDTCR,R17

 sei                        ;enable interrupts.

; ********************
; * Now the main bit *
; ********************
newframe:

 ser R19               ;Char line counter
 cpi R22,0            ;Inc frame counter if not 0
 breq ninc
 inc R22
ninc:

waitev:
 sbis PIND,VSYNC         ;Wait for VSync
 rjmp waitev

 out TCNT1H,R16           ;clear line counter.
 out TCNT1L,R16

; *********************************************************************
; *                                                                   *
; * Main blanking bit                                                 *
; *                                                                   *
; *********************************************************************

mainloop:
 sbis PIND,VSYNC         ;Sync on VSync if it happens.
 rjmp newframe

 sei
 wdr                 ;keep watchdog happy.

 sleep               ;wait for hsync pulses. This takes exactly the same
                     ;amount of time to detect the pulse every time.
                     ; takes 8 clock cycles to wake up propper (800ns @10MHz)

 cli                ;Ignore further fake pulses.
 
 in R17,TCNT1L       ;test to see which line we are on.
 in R18,TCNT1H       ;happens during HSync.
 mov R2,R17
 mov R3,R18          ;Store now to get rid of fake pulses during line.
                     ;1.3us after hsync up to here.
 cpi R18,0
 breq topofframe
 cpi R17,50           ;start lower 
 brmi mainloop
 cpi R17,54          ;end lower (55 gives 312 lines )
 brpl mainloop       
 rjmp lowerframe
topofframe:
 cpi R17,5           ;start upper (after post-equalizing pulses)
 brmi dontblank
 cpi R17,25          ;end upper
 brpl dontblank
 nop     
lowerframe:          ;2.1us after hsync up to here.

 cbi PORTD,BRIGHT     ;Blackish

 ;code above takes 2.2us
 ;Need to waste 4.8us - 2.2us = 2.6us to get to end of sync pulse.
 ldi R17,8    ;100ns
hswaste:
 dec R17       ;100ns
 brne hswaste  ;200ns
 nop

 sbi PORTB,BLANKOUT   ;Blank rest of video line.
 ldi R17,193          ;100ns  
bdlay:
 dec R17              ;100ns
 brne bdlay           ;200ns
 cbi PORTB,BLANKOUT   ;switch to normal video.
 
 out TCNT1H,R3           ;restore counter, dont count pulses during a line.
 out TCNT1L,R2
 
 rjmp mainloop

;******************************
;* Non essential nice bits    *
;******************************

dontblank:
 cpi R22,0           ;only put down message when appropriate.
 breq mainloop

 in R17,TCNT1L       ;test to see which line we are on.
 in R18,TCNT1H

 cpi R17,50          ;put message on lines 50-57
 brmi mainloop
 cpi R17,57
 brpl mainloop

 sbi PORTD,BRIGHT     ;whiteish.
 ldi R17,40           ;space from left to message.
dlay:
 dec R17
 brne dlay

 inc R19               ;char line counter
 ldi R28,Lookup
 clr R29

allmess:                 ;Get positions in character set lookup table.
 ld R0,Y+                ;this is what causes the spaces between chars.
 ld R1,Y+
 cp R1,R21
 breq mainloop
 add R0,R19
 adc R1,R16

 ldi R30,(smiley*2)&255   ;Load the start of the character set.
 ldi R31,(smiley*2)>8
 add R30,R0
 adc R31,R1
 
 lpm                     ;get byte from rom.
 
 out PORTB,R0            ;Blast out bit pattern on a pin.
 lsr R0
 out PORTB,R0
 lsr R0
 out PORTB,R0
 lsr R0
 out PORTB,R0
 lsr R0
 out PORTB,R0

 cbi PORTB,BLANKOUT

 rjmp allmess

smiley:
 .include "charset.asm"

message:
 .db 86,49,94,48,48,98,0

 

